@@ -4,15 +4,13 @@
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
-"""
-Remove raw HTML from README.rst to make it compatible with PyPI on
+"""Remove raw HTML from README.rst to make it compatible with PyPI on
 dist upload.
 """
 
 import argparse
 import re
 
-
 summary = """\
 Quick links
 ===========
@@ -37,7 +35,7 @@ Sponsors
 
 `Add your logo <https://github.com/sponsors/giampaolo>`__.
 
-Example usages"""  # noqa
+Example usages"""
 
 
 def main():
